﻿using Coscine.ApiCommons.Middleware;
using Coscine.ApiCommons.Utils;
using Coscine.Configuration;
using Coscine.Database.Settings;
using LinqToDB.Data;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.IdentityModel.Tokens;

namespace Coscine.ApiCommons
{
    public abstract class AbstractDefaultStartup
    {
        private string _basePath;
        private IConfiguration _configuration { get; set; }

        internal IConfiguration GetConfiguration()
        {
            return _configuration;
        }

        public virtual void SetBasePath(ApplicationInformation applicationInformation)
        {
            _basePath = $"/{applicationInformation.PathPrefix}";
            _configuration = new ConsulConfiguration();
        }

        public virtual void ConfigureServicesExtension(IServiceCollection services)
        {

        }

        public virtual void ConfigureServicesExtensionLate(IServiceCollection services)
        {

        }

        public virtual void ConfigureServices(IServiceCollection services)
        {
            ConfigureServicesExtension(services);

            services.AddMvc();

            ConfigureServicesExtensionLate(services);
        }

        public virtual void ConfigureExtension(IApplicationBuilder app, IHostingEnvironment env)
        {

        }

        public virtual void ConfigureExtensionLate(IApplicationBuilder app, IHostingEnvironment env)
        {

        }

        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            ConfigureExtension(app, env);

            app.UseDeveloperExceptionPage();

            app.UseMiddleware<LoggingMiddleware>();
            app.UseCors(builder => builder
            .AllowAnyOrigin()
            .AllowAnyMethod()
            .AllowAnyHeader()
            .AllowCredentials());

            app.UseAuthentication();

            app.UsePathBase(_basePath);
            app.UseMvc();

            app.UseStatusCodePages(async context =>
            {
                context.HttpContext.Response.ContentType = "text/plain";

                await context.HttpContext.Response.WriteAsync(
                    "Status code page, status code: " +
                    context.HttpContext.Response.StatusCode);
            });

            DataConnection.DefaultSettings = new CoscineSettings(_configuration);

            ConfigureExtensionLate(app, env);
        }
    }
}
