﻿using Coscine.ApiCommons.Utils;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.IdentityModel.Tokens;

namespace Coscine.ApiCommons
{
    /// <summary>
    /// This Startup applies the JWT token authentification additionally.
    /// </summary>
    public abstract class AbstractStartup : AbstractDefaultStartup
    {
        private JWTHandler _jWTHandler;

        public override void SetBasePath(ApplicationInformation applicationInformation)
        {
            base.SetBasePath(applicationInformation);
            _jWTHandler = new JWTHandler(GetConfiguration());
        }

        public override void ConfigureServices(IServiceCollection services)
        {
            ConfigureServicesExtension(services);

            services.AddMvc();

            var key = _jWTHandler.GetSecurityKey();
            services.AddAuthentication(x =>
            {
                x.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                x.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            })
            .AddJwtBearer(x =>
            {
                x.RequireHttpsMetadata = false;
                x.SaveToken = true;
                x.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuerSigningKey = true,
                    IssuerSigningKey = key,
                    ValidateIssuer = false,
                    ValidateAudience = false
                };
            });

            ConfigureServicesExtensionLate(services);
        }

    }
}
